"use strict";

// 信息脱敏

/**
 * 脱敏公用
 * @param str 脱敏字符串
 * @param begin 起始保留长度，从0开始
 * @param end 结束保留长度，到str.length结束
 * @returns {string}
 * desensitized.common('12345678901234',4, -3)     // '1234*******234'
 */
function common(str, beginLen, endLen) {
  var len = str.length;
  var firstStr = str.substr(0, beginLen);
  var lastStr = str.substr(endLen);
  var middleStr = str.substring(beginLen, len - Math.abs(endLen)).replace(/[\s\S]/ig, '*');
  tempStr = firstStr + middleStr + lastStr;
  return tempStr;
}
/**
 * 邮箱脱敏
 * @param str 脱敏字符串
 * @returns {string}
 * desensitized.email('hi@qq.com')==>'h***@qq.com'
 */


function email(str) {
  if (!str) {
    return "";
  }

  var avg;
  var splitted;
  var email1;
  var email2;
  splitted = str.split('@');
  email1 = splitted[0];
  avg = email1.length / 2;
  email1 = email1.substring(0, email1.length - avg);
  email2 = splitted[1];
  return email1 + '***@' + email2; // 输出为11223***@qq.com
}
/**
 * 银行卡脱敏
 * @param str 脱敏字符串
 * @returns {string}
 * desensitized.bankCard('6212268202014638888')==>'6212 **** **** 8888'
 */


function bankCard(str) {
  if (!str) {
    return "";
  }

  if (Object.prototype.toString.call(str).slice(8, -1) !== 'Number' && Object.prototype.toString.call(str).slice(8, -1) !== 'String') {
    return '';
  }

  if (Object.prototype.toString.call(str).slice(8, -1) === 'Number') {
    str = String(str);
  }

  return str.replace(/(?<=\d{4})\d+(?=\d{4})/, " **** **** ");
}
/**
 * @desc 手机号脱敏
 * @param  {str}  string
 * @return string
 */


function phone(str) {
  // 手机号正则校验
  var phoneReg = /^(13[0-9]|14[01456879]|15[0-35-9]|16[2567]|17[0-8]|18[0-9]|19[0-35-9])\d{4}(\d{4})$/;

  if (!str) {
    return "";
  }

  return str.replace(phoneReg, '$1****$2');
}
/**
 * @desc 固定电话脱敏
 * @param  {str}  string
 * @return string
 */


function tel(str) {
  if (!str) {
    return str;
  }

  ;
  var newTel;
  var telArr = str.split(/[-,_\s]/);

  if (telArr.length > 1) {
    newTel = telArr.map(function (num) {
      if (!num) {
        return '';
      }

      if (num.length <= 4) {
        var stars = new Array(num.length).join("*");
        return num.replace(num.substring(1, num.length), stars);
      }

      if (num.length >= 5) {
        var _stars = new Array(num.length - 2).join("*");

        return num.replace(num.substring(3, num.length), _stars);
      }
    }).join('-');
  } else {
    var stars = new Array(str.length - 2).join("*");
    newTel = str.replace(str.substring(3, str.length), stars);
  }

  return newTel;
}
/**
 * @desc 身份证脱敏
 * @param  {str}  string
 * @return string
 */


function idNo(str) {
  // 简易身份证号码正则
  var idNoReg = /(\d{6})(\d*)(\w{3})/; //

  if (!str) {
    return "";
  }

  if (idNoReg.test(str)) {
    var text1 = RegExp.$1;
    var text3 = RegExp.$3;
    var text2 = RegExp.$2.replace(/./g, "*");
    return text1 + text2 + text3;
  }

  return str;
}
/**
 * @desc 地址脱敏
 * @param  {str}  string
 * @return string
 */


function address(str) {
  // 地址正则
  var addrReg = /(.{9})(.*)/;

  if (!str) {
    return "";
  }

  if (addrReg.test(str)) {
    var text1 = RegExp.$1;
    var text2 = RegExp.$2.replace(/./g, "*");
    return text1 + text2;
  }

  return str;
}
/**
 * 姓名脱敏 保留首尾、末尾
 * @param str
 * @returns {string}
 * console.log(formatName('杨倩倩倩'));    //杨**倩
 */


function name(str) {
  if (str.length === 2) {
    return str.substr(0, 1) + '*';
  } else if (str.length > 2) {
    var _char = '';

    for (var i = 0, len = str.length - 2; i < len; i++) {
      _char += '*';
    }

    return str.substr(0, 1) + _char + str.substr(-1, 1);
  }
}

var desensitizedUtils = {
  common: common,
  email: email,
  bankCard: bankCard,
  phone: phone,
  tel: tel,
  idNo: idNo,
  address: address,
  name: name
};
module.exports = desensitizedUtils;