"use strict";

/**
 * 将扁平数组转换为树形对象
 * 适用于来自同一个表的数据，即idName的数据不重复
 *
 * @param {*} flatData 同级数组数据
 * @param {option}
 *        idName 唯一 id 名称
 *        pidName 父级 id 名称
 *        childrenName 层级列表key名称
 * @returns 树形结构数组数据
 */
function flatToTree(flatData, _ref) {
  var idName = _ref.idName,
      pidName = _ref.pidName,
      _ref$childrenName = _ref.childrenName,
      childrenName = _ref$childrenName === void 0 ? 'children' : _ref$childrenName;
  var treeData = [];

  if (!Array.isArray(flatData)) {
    return treeData;
  }

  flatData.forEach(function (item) {
    delete item[childrenName];
  });
  var map = {};
  flatData.forEach(function (item) {
    // 深拷贝，该方式将使该转换方法失效
    // map[item[idName]] = JSON.parse(JSON.stringify(item));
    // 浅拷贝，将对 item 的引用传递给 map
    map[item[idName]] = item;
  });
  flatData.forEach(function (item) {
    // 深拷贝，该方式将使该转换方法失效
    // let parent = JSON.parse(JSON.stringify(map[item[pidName]]));
    // 浅拷贝，将对 map 的引用传递给 parent
    var parent = map[item[pidName]];

    if (parent) {
      // 利用了浅拷贝的引用传递，最终 flatData 中的 item 将会改变
      (parent[childrenName] || (parent[childrenName] = [])).push(item);
    } else {
      treeData.push(item);
    }
  });
  return treeData;
}

module.exports = flatToTree;