import { ColorInput, ConstructorOptions, Instance } from './color'

declare module "@tnt-temp/utils" {
    // browser
    export function getBrowserName():string;
    export function getBrowserNameVersion():string;

    // color
    export function tinyColor(color: ColorInput, opt: ConstructorOptions): Instance;

    // common
    export namespace base64 {
        function encode(input: string, binary?: BinaryType): string;
        function decode(input: string, binary?: BinaryType): string;
        function _utf8_encode(str: string): string;
        function _utf8_decode(str: string): string;
    }
    export function bytesToSize(num: number): string;
    export function compareVersion(version1: string, version2: string): number;
    export function copy(text: string, options?: {debug?:boolean,message?:string,format?:string,onCopy?:function}): void;
    export function deleteEmptyObjItem(obj: object): object;
    export function generateUUID(): any;
    export function getExtname(str: string): string;
    export function getStrLength(str: string): number;
    export function getUrlKey(str1: string, href: string): string;
    export namespace queryString {
        function parse(query: string, options?: any): object;
        function parseUrl(query: string, options?: any): object;
        function stringify(obj: object, options?: any): string;
    }
    export function safeParseJSON(str: string, defaultObj:Object|Array<[]>): any;
    export function saveAs(blob: Blob|string, fileName: string): void;
    export function trim(str: string, obj: string | object): string;

    // convert
    export function base64ToBlob(base64: string): Blob;
    export function base64ToFile(base64: string, fileName?:string): File;
    export function blobToFile(blob: Blob,fileName?:string): File;
    export function flatToTree(flatData: Array<[]>,option:{idName:string,pidName:string,childrenName?:string}): Array<[]>;
    export function numberToChinese(number: number): string|null;
    export function numberToCnMoney(number: number): string|null;
    export function parseQueryString(str: string): object;
    export function pinyin(str:string, separator?:string, lowerCase?:boolean): string;
    export function toThousands(num: number): string;
    export function treeToFlat(treeData: Array<[]>,childrenName?:string): Array<[]>;

    // cookie
    export function getCookie(str: string): string;
    export function removeCookie(str: string): void;
    export function setCookie(str1: string, str2: string, date: number): void;

    // functional
    export function debounce(fn:function, wait:number, option?:{ leading?:boolean, maxWait?:number, trailing?:boolean }): function;
    export function throttle(fn:function, wait:number, option?:{ leading?:boolean, trailing?:boolean }): function;

    // judge
    export function isJSON(str: string): boolean;
    export function isEmptyObject(obj: object): boolean;

    // operation
    export function accAdd(num1: number, num2: number): number;
    export function accDiv(num1: number, num2: number): number;
    export function accMul(num1: number, num2: number): number;
    export function accSub(num1: number, num2: number): number;

    // random
    export function randomColor(): string;
    export function randomNum(min: number,max:number): number;

    // series
    export namespace desensitizedUtils {
        function common(str: string, begin:number, end:number): string;
        function email(str: string): string;
        function bankCard(str: string): string;
        function phone(str: string): string;
        function tel(str: string): string;
        function idNo(str: string): string;
        function address(str: string): string;
        function name(str: string): string;
    }

    export namespace regexpUtils {
        function isColor(str: string): boolean;
        function isEmail(str: string): boolean;
        function isIdCard(str: string | number): boolean;
        function isPhoneNum(str: string | number): boolean;
        function isUrl(str: string): boolean;
        function isDomain(str: string): boolean;
        function isIPv4(str: string): boolean;
        function isIPv6(str: string): boolean;
        function isBankNumber(str: string): boolean;
        function isCnName(str: string): boolean;
        function isEnName(str: string): boolean;
        function isCarNumber(str: string): boolean;
        function isPassportNumber(str: string): boolean;
        function isPostNumber(str: string): boolean;
    }

    export namespace typeUtils {
        function getType(obj: any): 'String'|'Number'|'Boolean'|'Function'|'Null'|'Undefined'|'Object'|'Array'|'Date'|'RegExp'|'Error'|'Symbol'|'Promise'|'Set';
        function isEmpty(obj: any): string;
        function isString(p: any): boolean;
        function isNumber(p: any): boolean;
        function isBoolean(p: any): boolean;
        function isFunction(p: any): boolean;
        function isNull(p: any): boolean;
        function isUndefined(p: any): boolean;
        function isObject(p: any): boolean;
        function isArray(p: any): boolean;
        function isDate(p: any): boolean;
        function isRegExp(p: any): boolean;
        function isError(p: any): boolean;
        function isSymbol(p: any): boolean;
        function isPromise(p: any): boolean;
        function isSet(p: any): boolean;
        function isFalse(p: any): boolean;
        function isTrue(p: any): boolean;
        function isPC(): boolean;
        function isMobile(): boolean;
        function isDarkMode(): boolean;
        function isIOS(): boolean;
        function isAndroid(): boolean;
    }

    // storage
    export namespace safeStorage {
        function setItem(key: string, value: any): void;
        function getItem(key: string, value: any): any;
        function removeItem(key: string): void;
    }
}
