"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isEmpty = void 0;

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/**
 * @desc   获取变量准确类型
 * @param  obj any
 * @return string===>'String'|'Number'|'Boolean'|'Function'|'Null'|'Undefined'|'Object'|'Array'|'Date'|'RegExp'|'Error'|'Symbol'|'Promise'|'Set'
 */
function getType(obj) {
  var type = _typeof(obj);

  if (type !== 'object') {
    return type;
  } //如果不是object类型的数据，直接用typeof就能判断出来
  //如果是object类型数据，准确判断类型必须使用 Object.prototype.toString.call(obj) 的方式才能判断


  return Object.prototype.toString.call(obj).replace(/^\[object (\S+)\]$/, '$1');
}
/**
 * 判断元素是否为空
 * @param value
 */


var isEmpty = function isEmpty(value) {
  if (value == null) return true;
  if (isBoolean(value)) return !value;
  if (isNumber(value)) return !value;
  if (value instanceof Error) return value.message === "";

  switch (getType(value)) {
    case "function":
    case "string":
    case "array":
      {
        return !value.length;
      }
  }

  if (_typeof(value) === "object") {
    if (hasIterable && typeof value[Symbol.iterator] === "function") {
      var size = 0;
      value.forEach(function () {
        return size++;
      });
      return !size;
    }

    return !Object.keys(value).length;
  }

  return false;
};
/**
 * @desc 判断是否字符串
 * @param any
 * @return {Boolean}
 */


exports.isEmpty = isEmpty;

function isString(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'String';
}
/**
 *
 * @desc 判断是否数字
 * @param any
 * @return {Boolean}
 */


function isNumber(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Number';
}
/**
 *
 * @desc 是否boolean
 * @param any
 * @return {Boolean}
 */


function isBoolean(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Boolean';
}
/**
 *
 * @desc 判断是否函数
 * @param any
 * @return {Boolean}
 */


function isFunction(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Function';
}
/**
 *
 * @desc 判断是否为null
 * @param any
 * @return {Boolean}
 */


function isNull(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Null';
}
/**
 *
 * @desc 判断是否undefined
 * @param any
 * @return {Boolean}
 */


function isUndefined(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Undefined';
}
/**
 *
 * @desc 判断是否对象
 * @param any
 * @return {Boolean}
 */


function isObject(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Object';
}
/**
 *
 * @desc 判断是否数组
 * @param any
 * @return {Boolean}
 */


function isArray(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Array';
}
/**
 *
 * @desc 判断是否时间
 * @param any
 * @return {Boolean}
 */


function isDate(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Date';
}
/**
 *
 * @desc 判断是否正则
 * @param any
 * @return {Boolean}
 */


function isRegExp(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'RegExp';
}
/**
 *
 * @desc 判断是否错误对象
 * @param any
 * @return {Boolean}
 */


function isError(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Error';
}
/**
 *
 * @desc 判断是否Symbol函数
 * @param any
 * @return {Boolean}
 */


function isSymbol(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Symbol';
}
/**
 *
 * @desc 判断是否Promise对象
 * @param any
 * @return {Boolean}
 */


function isPromise(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Promise';
}
/**
 *
 * @desc 判断是否Set对象
 * @param any
 * @return {Boolean}
 */


function isSet(o) {
  return Object.prototype.toString.call(o).slice(8, -1) === 'Set';
}
/**
 *
 * @desc 判断是否是false
 * @param any
 * @return {Boolean}
 */


function isFalse(o) {
  if (o == '' || o == undefined || o == null || o == 'null' || o == 'undefined' || o == 0 || o == false || o == NaN) return true;
  return false;
}

function isTrue(o) {
  return !isFalse(o);
}
/**
 *
 * @desc 判断是否为 PC 端
 * @return {Boolean}
 */


function isPC() {
  var userAgentInfo = navigator.userAgent;
  var Agents = ["Android", "iPhone", "SymbianOS", "Windows Phone", "iPad", "iPod"];
  var flag = true;

  for (var v = 0; v < Agents.length; v++) {
    if (userAgentInfo.indexOf(Agents[v]) > 0) {
      flag = false;
      break;
    }
  }

  return flag;
}
/**
 *
 * @desc 判断是否为 mobile 端
 * @return {Boolean}
 */


function isMobile() {
  var mobile = navigator.userAgent.match(/(phone|pad|pod|iPhone|iPod|ios|iPad|Android|Mobile|BlackBerry|IEMobile|MQQBrowser|JUC|Fennec|wOSBrowser|BrowserNG|WebOS|Symbian|Windows Phone)/i);
  return mobile != null;
}
/**
 *
 * @desc 判断浏览器是否是dark模式
 * @return {Boolean}
 */


function isDarkMode() {
  return window.matchMedia('(prefers-color-scheme: dark)').matches;
}
/**
 *
 * @desc 判断是否IOS
 * @return {Boolean}
 */


function isIOS() {
  return /ios|iphone|ipad|ipod/.test(navigator.userAgent.toLowerCase());
}
/**
 *
 * @desc 判断是否安卓
 * @return {Boolean}
 */


function isAndroid() {
  return /android/.test(navigator.userAgent.toLowerCase());
}

var typeUtils = {
  getType: getType,
  isEmpty: isEmpty,
  isString: isString,
  isNumber: isNumber,
  isBoolean: isBoolean,
  isFunction: isFunction,
  isNull: isNull,
  isUndefined: isUndefined,
  isObject: isObject,
  isArray: isArray,
  isDate: isDate,
  isRegExp: isRegExp,
  isError: isError,
  isSymbol: isSymbol,
  isPromise: isPromise,
  isSet: isSet,
  isFalse: isFalse,
  isTrue: isTrue,
  isPC: isPC,
  isMobile: isMobile,
  isDarkMode: isDarkMode,
  isIOS: isIOS,
  isAndroid: isAndroid
};
module.exports = typeUtils;